<?php
namespace WowGmbh\AmazonPay\Utility;

/***************************************************************
 *
 *  Copyright notice
 *
 *  (c) 2018 WOW-GmbH <info@wow-gmbh.de>, WOW-GmbH.de
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Utility\GeneralUtility;
use \WowGmbh\AmazonPay\Domain\Model\Payment;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * Tools
 *
 * @package amazon_pay
 * @subpackage Utility
 */
class Tools
{

    /**
     * Gets a template path from configuration[view]
     * @param array $configurationView Pass $configuration['view'] in this
     * @param string $type Type of template path (for example: template, partial or layout)
     * @return string
     */
    public static function getTemplatePath($configurationView, $type = 'template', $file)
    {
        $path = '';
        $rootPath = $type . 'RootPath';
        $rootPaths = $type . 'RootPaths';
        // Are there path's' available?!
        if (isset($configurationView[$rootPaths]) && !empty($configurationView[$rootPaths])) {
            // first, sort by key
            krsort($configurationView[$rootPaths]);
            $keys = array_keys($configurationView[$rootPaths]);
            // find the first Template in array
            for ($i = 0; $i < count($keys); $i++) {
                // Template really exists?
                $path = GeneralUtility::getFileAbsFileName($configurationView[$rootPaths][$keys[$i]]);
                if (file_exists($path . $file)) {
                    break;
                }
            }
        } else {
            if (isset($configurationView[$rootPath])) {
                //var_dump($configuration, $type, $file);
                $path = GeneralUtility::getFileAbsFileName($configurationView[$rootPath]);
                if (file_exists($path . $file)) {
                    // path found, everything's alright!
                } else {
                    throw new \Exception('Tools::getTemplatePath (without s) -> path file not exist');
                }
            }
        }
        if ($path == '') {
            throw new \Exception('Tools::getTemplatePath not found (' . serialize($configurationView) . ', ' . $type . ', ' . $file . ')');
        }
        return $path;
    }

    /**
     * @param string $code
     * @param bool $debug
     * @return NULL|string
     */
    static public function translateErrorCode($code='', $debug=false) {
        $codeLCU = GeneralUtility::camelCaseToLowerCaseUnderscored($code);
        $translationKey = 'tx_amazonpay_message.' . $codeLCU;
        $translation = LocalizationUtility::translate($translationKey, 'AmazonPay');
        if(!$translation) {
            $translation = 'Translate: ' . $code;
            self::adminEmail('Amazon-Pay: Missing translation', $translation);
        }
        if($debug) {
            $translation .= ' (' . $code . ')';
        }
        return $translation;
    }

    /**
     * @param string $subject
     * @param string $message
     * @return int
     */
    static public function adminEmail($subject='Amazon-Pay', $message='') {
        $emailData = array(
            'fromName' => 'Amazon-Pay',
            'fromEmail' => 'info@wow-gmbh.de',
            'toName' => 'Amazon-Pay',
            'toEmail' => 'info@wow-gmbh.de'
        );
        /** @var \TYPO3\CMS\Core\Mail\MailMessage $mail */
        $mail = GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\Mail\\MailMessage');
        $mail->setFrom(array($emailData['fromEmail'] => $emailData['fromName']));
        $mail->setTo(array($emailData['toEmail'] => $emailData['toName']));
        $mail->setSubject($subject);
        $mail->setBody($message);
        return $mail->send();
    }

    /**
     * @param array $array
     * @param bool $debug
     * @return NULL|string
     */
    static public function processError($array, $debug=false) {
        $originalMessage = $array['Message'];
        $array['Message'] = self::translateErrorCode($array['Code'], $debug);
        if($debug) {
            $array['Message'] .= ' (' . $originalMessage . ')';
        }
        return $array;
    }

    /**
     * Translate constraint messages
     * @param string $code
     * @param string $message
     * @param bool $debug
     * @return NULL|string
     */
    static public function processConstraintMessage($code='', $message='', $debug=false) {
        $codeLCU = GeneralUtility::camelCaseToLowerCaseUnderscored($code);
        $translationKey = 'tx_amazonpay_message.' . $codeLCU;
        $translation = LocalizationUtility::translate($translationKey, 'AmazonPay');
        if(!$translation) {
            $translation = 'Translate: ' . $code;
            self::adminEmail('Amazon-Pay: Missing translation', $translation);
        }
        if($debug) {
            $translation .= ' (' . $message . ' - ' . $code . ')';
        }
        return $translation;
    }

    /**
     * Log a emitted signal
     * @param array $data
     * @param \WowGmbh\AmazonPay\Controller\AmazonController $pObj
     */
    public function logAfterAmazonPaymentSuccessfulSlot($data=array(), $pObj) {
        /** @var \WowGmbh\AmazonPay\Domain\Repository\LogRepository $logRepository */
        $paymentRepository = GeneralUtility::makeInstance('WowGmbh\\AmazonPay\\Domain\\Repository\\PaymentRepository');
        /** @var \WowGmbh\AmazonPay\Domain\Model\Log $log */
        $log = GeneralUtility::makeInstance('WowGmbh\\AmazonPay\\Domain\\Model\\Log');
        // General data
        $log->setAmazonOrderReferenceId($data['orderReferenceId']);
        $log->setAction('receiveAmazonPaymentSuccessfulSignal');
        // Get the related payment object
        $payment = $paymentRepository->findOneByAmazonOrderReferenceId($data['orderReferenceId']);
        if($payment instanceof Payment) {
            $log->setPayment($payment);
        }
        // Request
        $log->setRequest(serialize($data));
        $log->setRequestText(self::arrayToString($data));
        // Write into log
        /** @var \WowGmbh\AmazonPay\Domain\Repository\LogRepository $logRepository */
        $logRepository = GeneralUtility::makeInstance('WowGmbh\\AmazonPay\\Domain\\Repository\\LogRepository');
        $logRepository->add($log);
    }

    /**
     * Converts an array to string
     * @param $array
     * @return string
     */
    public static function arrayToString($array) {
        ini_set('xdebug.overload_var_dump', 0);
        ob_start();
        var_dump($array);
        $varDump = ob_get_clean();
        $varDump = strip_tags($varDump);
        $varDump = htmlspecialchars_decode($varDump);
        return $varDump;
    }

}