/**
 * 
 * @author: WOW-GmbH
 */
var amazonPayment = {

	/**
	 * Debug ths library
	 * @type boolean
	 */
	debug: false,
	
	/**
	 * Configuration
	 * @type Object
	 */
	config: {},

	/**
	 * Current login state
	 * @type boolean
	 */
	loggedIn: false,

	/**
	 * This object is already initialized?
	 * @type boolean
	 */
	initialized: false,

	/**
	 * Payment confirmed?
	 * @type boolean
	 */
	confirmed: false,

	/**
	 * Object with cookie data
	 * @type Object
	 */
	cookieData: {},

	/**
	 * Object with state data
	 * @type Object
	 */
	stateData: {},

	/**
	 * amazon.Login.authorize
	 * @type Object
	 */
	authRequest: null,

	/**
	 * Stores the address consent toke
	 * @type string
	 */
	addressConsentToken: '',

	/**
	 * Stores the current order reference id
	 * @type string
	 */
	orderReferenceId: '',

	/**
	 * Address book widget
	 * @type Object
	 */
	addressBook: null,
	
	/**
	 * Wallet widget
	 * @type Object
	 */
	wallet: null,

	/**
	 * Initialize Payment library
	 * @param config
	 */
	initialize: function(config) {
		// Execute only, if not already initialized
		if(!this.initialized) {
			// Debugging is activated?!
			if(config.debug) {
				this.debug = config.debug;
				Logger.enableLogger();
			}
			Logger.log('amazonPayment.initialize:', config);
			// Remind as initialized
			this.initialized = true;
			// Remind passed configuration
			this.config = config;
			/**
			 * @todo Region: us,de,uk,jp
			 */
			// User is already logged in?
			//if(typeof amazonPayment.getCookieValue('amazon_Login_state_cache') != 'undefined') {
				// amazonPayment.setLoggedIn(true);
				// amazonPayment.getStateData();
				// Don't use an existing login -> logout!
				amazonPayment.logout();
			//}
			//else {
			//	// Show/hide some elements
			//	jQuery('.amazon-show-on-logout').show();
			//	jQuery('.amazon-hide-on-logout').hide();
			//}
			// Bind back to previous page
			var backToPreviousPageButton = jQuery('#amazon-back-to-previous-page');
			if(backToPreviousPageButton.length>0) {
				backToPreviousPageButton.click(amazonPayment.backToPreviousPage);
			}
			// Bind cancel button
			var logoutButton = jQuery('#amazon-cancel-and-back-to-previous-page');
			if(logoutButton.length>0) {
				logoutButton.click(amazonPayment.cancelPayment);
			}
			// Bind confirm payment button
			var confirmPaymentButton = jQuery('#amazon-confirm-payment');
			if(confirmPaymentButton.length>0) {
				confirmPaymentButton.click(amazonPayment.confirmPayment);
				confirmPaymentButton.prop('disabled', true);
			}
			// Bind modify payment button
			var modifyPaymentButton = jQuery('#amazon-modify-payment');
			if(modifyPaymentButton.length>0) {
				modifyPaymentButton.click(amazonPayment.modifyPayment);
			}
			// Bind finish payment button
			var finishPaymentButton = jQuery('#amazon-finish-payment');
			if(finishPaymentButton.length>0) {
				finishPaymentButton.click(amazonPayment.finishPayment);
			}
		}
		else {
			Logger.warn('amazonPayment.initialize: multiple initialization blocked', config);
		}
	},

	/**
	 * Get a cookie value
	 * @param name
	 * @param c
	 * @param C
	 * @param i
	 * @returns {*}
	 */
	getCookieValue: function(name, c, C, i){
		if(amazonPayment.cookieData.length>0) {
			return amazonPayment.cookieData[name];
		}
		c = document.cookie.split('; ');
		amazonPayment.cookieData = {};
		for(i=c.length-1; i>=0; i--){
			C = c[i].split('=');
			amazonPayment.cookieData[C[0]] = C[1];
		}
		return amazonPayment.cookieData[name];
	},
	
	getStateData: function() {
		amazonPayment.stateData = decodeURIComponent(amazonPayment.getCookieValue('amazon_Login_state_cache'));
		amazonPayment.stateData = JSON.parse(amazonPayment.stateData);
		Logger.log(amazonPayment.stateData);
	},

	/**
	 * Set the login state
	 * @param login
	 */
	setLoggedIn: function(login) {
		Logger.log('setLoggedIn: ', login);
		amazonPayment.loggedIn = login;
		// Show/hide login button
		if(amazonPayment.loggedIn) {
			jQuery('.amazon-show-on-login').show();
			jQuery('.amazon-hide-on-login').hide();
		}
		else  {
			jQuery('.amazon-show-on-login').hide();
			jQuery('.amazon-hide-on-login').show();
		}
	},

	/**
	 * Click on back to previous page button
	 */
	backToPreviousPage: function() {
		amazonPayment.toggleLoadingInfo();
		amazonPayment.logout(true);
	},

	/**
	 * Perform a logout
	 */
	logout: function(backToPreviousPage) {
		Logger.log('logout');
		// Show/hide some elements
		jQuery('.amazon-show-on-logout').show();
		jQuery('.amazon-hide-on-logout').hide();
		amazonPayment.loggedIn = false;
		if(typeof amazon !== 'undefined') {
			amazon.Login.logout();
		}
		// Clear all wrapper
		jQuery('#amazon-address-book').html('');
		jQuery('#amazon-wallet').html('');
		jQuery('#amazon-confirmation').html('');
		// Unset data
		amazonPayment.addressConsentToken = '';
		amazonPayment.orderReferenceId = '';
		// Back to previous page
		if(typeof backToPreviousPage === 'boolean' && backToPreviousPage) {
			window.history.back();
		}
	},

	/**
	 * Initializes the payment in backend
	 */
	initializePayment: function() {
		amazonPayment.scrollTo();
		// Prepare API url
		//Logger.log(amazonPayment.config.apiUri.replace('REFERENCE_ID', amazonPayment.orderReferenceId));
		// Prepare parameter
		var parameter = {
			tx_amazonpay_jsonapi: {
				orderReferenceId: amazonPayment.orderReferenceId,
				addressConsentToken: amazonPayment.addressConsentToken,
				paymentPresetUid: parseInt(amazonPayment.config.paymentPresetUid, 10),
				paymentAction: 'initialize'
			}
		};
		Logger.log('initializePayment parameter:', parameter);
		// Send payment initialization
		jQuery.ajax({
			url: amazonPayment.config.apiUri,
			method: 'POST',
			data: parameter,
			dataType: 'json',
			success: function(data) {
				Logger.log('initializePayment.success', data);
				// Hide loading info
				amazonPayment.toggleLoadingInfo();
				// Some errors happened?
				if(typeof data.error !== 'undefined') {
					Logger.error('initializePayment.success failed: ', data.error);
					var message = data.error.Message;
					if(amazonPayment.debug) {
						message += ' (' + data.error.Code + ')';
					}
					FlashMessage.push(message, 'danger', '#amazon-flash-messages');
					if(data.error.Code === 'SignatureDoesNotMatch') {
						amazonPayment.logout();
					}
				}
				else {
					amazonPayment.modifyPayment();
				}
			},
			error: function() {
				var message = 'Payment initialization failed! Please contact the administrator of this website!';
				FlashMessage.push(message, 'danger', '#amazon-flash-messages');
				amazonPayment.toggleLoadingInfo();
				amazonPayment.logout();
			}
		});
	},

	/**
	 * Confirm payment settings
	 */
	confirmPayment: function() {
		amazonPayment.scrollTo();
		amazonPayment.toggleLoadingInfo();
		// Prepare parameter
		var parameter = {
			tx_amazonpay_jsonapi: {
				orderReferenceId: amazonPayment.orderReferenceId,
				paymentAction: 'confirm',
				addressConsentToken: amazonPayment.addressConsentToken
			}
		};
		// Send payment initialization
		jQuery.ajax({
			url: amazonPayment.config.apiUri,
			method: 'POST',
			data: parameter,
			dataType: 'json',
			success: function(data) {
				var message = '';
				Logger.log('confirm done', data);
				// Error message available?
				if(typeof data['error'] !== 'undefined') {
					FlashMessage.push(data.error.Message, 'danger', '#amazon-flash-messages');
				}
				else {
					// Some constraints :(
					if (typeof data.orderReferenceDetails.constraints !== 'undefined') {
						// Display constraints as flash message
						for (var constraint in data.orderReferenceDetails.constraints) {
							if (data.orderReferenceDetails.constraints.hasOwnProperty(constraint)) {
								message = data.orderReferenceDetails.constraints[constraint];
								FlashMessage.push(message, 'danger', '#amazon-flash-messages');
							}
						}
					}
					// No constraints
					else {
						// Mark as confirmed
						amazonPayment.confirmed = true;
						// Decode html
						var html = window.atob(data.html);
						data.html = decodeURIComponent(escape(html));
						// Append html
						jQuery('#amazon-confirmation').html(data.html);
						// Show confirm html
						jQuery('.amazon-show-on-confirm').show();
						jQuery('.amazon-hide-on-confirm').hide();
						message = 'Please check you payment data and click finish payment!';
						FlashMessage.push(message, 'success', '#amazon-flash-messages');
					}
				}
				amazonPayment.toggleLoadingInfo();
			},
			error: function() {
				var message = 'Payment confirmation failed! Please contact the administrator of this website!';
				FlashMessage.push(message, 'danger', '#amazon-flash-messages');
				amazonPayment.toggleLoadingInfo();
				amazonPayment.logout();
			}
		});
	},

	/**
	 * Perform action, when customer want to modify his payment
	 */
	modifyPayment: function() {
		// Mark as not confirmed
		amazonPayment.confirmed = false;
		amazonPayment.scrollTo();
		jQuery('.amazon-show-on-modify').show();
		jQuery('.amazon-hide-on-modify').hide();
	},
	
	/**
	 * Finish payment process
	 */
	finishPayment: function() {
		// Finish only when the order reference is confirmed
		if(amazonPayment.confirmed) {
			amazonPayment.scrollTo();
			amazonPayment.toggleLoadingInfo();
			// Prepare parameter
			var parameter = {
				tx_amazonpay_jsonapi: {
					orderReferenceId: amazonPayment.orderReferenceId,
					paymentAction: 'finish',
					addressConsentToken: amazonPayment.addressConsentToken
				}
			};
			// Send payment initialization
			jQuery.ajax({
				url: amazonPayment.config.apiUri,
				method: 'POST',
				data: parameter,
				dataType: 'json',
				success: function(data) {
					amazonPayment.toggleLoadingInfo();
					Logger.log('finish done', data);
					// Error message available?
					if(typeof data['error'] !== 'undefined') {
						FlashMessage.push(data.error.Message, 'danger', '#amazon-flash-messages');
						// In case of amazon rejected -> logout!
						if(data.error.Code === 'ProcessingFailure' ||
							data.error.Code === 'TransactionTimedOut') {
							Logger.log('finishing failed because of: ' + data.error.Code, data);
							amazonPayment.cancelPayment(true);
						}
						else if(data.error.Code === 'AmazonRejected') {
							// Amazon is performing a cancellation by its own
							amazonPayment.logout();
						}
						else if(data.error.Code === 'InvalidPaymentMethod') {
							Logger.log('finishing failed because of: ' + data.error.Code, data);
							// Hide address boo widget
							jQuery('#amazon-address-book').hide();
							jQuery('#amazon-address-book-wrapper').hide();
							// Refresh wallet widget
							amazonPayment.wallet.bind('amazon-wallet');
							// Switch back to modify payment
							amazonPayment.modifyPayment();
						}
					}
					else {
						// Some constraints :(
						if(typeof data.orderReferenceDetails.constraints !== 'undefined') {
							Logger.log('finishing failed because of constraints: ', data.orderReferenceDetails.constraints);
							// Display constraints as flash message
							for (var constraint in data.orderReferenceDetails.constraints) {
								if(data.orderReferenceDetails.constraints.hasOwnProperty(constraint)) {
									message = data.orderReferenceDetails.constraints[constraint];
									FlashMessage.push(message, 'danger', '#amazon-flash-messages');
								}
							}
							// Switch back to modify payment
							amazonPayment.modifyPayment();
						}
						// No constraints
						else {
							FlashMessage.push(data.message, 'success', '#amazon-flash-messages');
							amazonPayment.logout();
							jQuery('.amazon-show-on-finish').show();
							jQuery('.amazon-hide-on-finish').hide();
						}
					}
				},
				error: function() {
					var message = 'Payment finishing failed! Please contact the administrator of this website!';
					FlashMessage.push(message, 'danger', '#amazon-flash-messages');
					amazonPayment.toggleLoadingInfo();
					amazonPayment.logout();
				}
			});
		}
		else {
			FlashMessage.push('Please confirm your payment!', 'danger', '#amazon-flash-messages');
		}
	},

	cancelPaymentSilent: false,

	/**
	 * Cancel payment process
	 */
	cancelPayment: function(silent) {
		amazonPayment.scrollTo();
		// Prepare parameter
		var parameter = {
			tx_amazonpay_jsonapi: {
				orderReferenceId: amazonPayment.orderReferenceId,
				paymentAction: 'cancel',
				addressConsentToken: amazonPayment.addressConsentToken
			}
		};
		if(typeof silent === 'boolean' && silent) {
			amazonPayment.cancelPaymentSilent = silent;
		}
		else {
			amazonPayment.toggleLoadingInfo();
		}
		// Send payment initialization
		jQuery.ajax({
			url: amazonPayment.config.apiUri,
			method: 'POST',
			data: parameter,
			dataType: 'json',
			success: function(data) {
				Logger.log('cancel done', data);
				amazonPayment.hideLoadingInfo();
				// Some errors happened?
				if(typeof data.error !== 'undefined' ) {
					FlashMessage.push(data.error.Message, 'danger', '#amazon-flash-messages');
				}
				else {
					FlashMessage.push(data.message, 'success', '#amazon-flash-messages');
					// If we use a silent cancelation, we dont redirect to the parent page!
					amazonPayment.logout(!amazonPayment.cancelPaymentSilent);
					jQuery('.amazon-show-on-finish').show();
					jQuery('.amazon-hide-on-finish').hide();
				}
			},
			error: function() {
				var message = 'Payment cancellation failed! Please contact the administrator of this website!';
				FlashMessage.push(message, 'danger', '#amazon-flash-messages');
				amazonPayment.toggleLoadingInfo();
				amazonPayment.logout();
			}
		});
	},

	/**
	 * On order reference
	 * @param orderReference
	 */
	onOrderReferenceCreate: function(orderReference) {
		amazonPayment.orderReferenceId = orderReference.getAmazonOrderReferenceId();
		Logger.log('amazonPayment.onOrderReferenceCreate orderReferenceId:', amazonPayment.orderReferenceId);
		// Initiate the payment in backend
		amazonPayment.initializePayment();
	},

	/**
	 * Error handling
	 * @param error
	 */
	onError: function (error) {
		Logger.log('addressBook.onAddressBookError', error);
		if(typeof error === 'object') {
			if(typeof error.getErrorMessage === 'function') {
				var message = error.getErrorMessage();
				if(amazonPayment.debug) {
					message += ' (' + error.getErrorCode() + ')';
				}
				FlashMessage.push(message, 'danger', '#amazon-flash-messages');
			}
		}
	},

	/**
	 * Show the address book widget
	 */
	showAddressBook: function () {
		if(jQuery('#amazon-address-book')) {
			Logger.log('showAddressBook: ', amazonPayment.orderReferenceId);
			var addressBookParameter = {
				sellerId: amazonPayment.config.sellerId,
				scope: amazonPayment.config.login.scope,
				design: {
					designMode: 'responsive'
				},
				onOrderReferenceCreate: amazonPayment.onOrderReferenceCreate,
				onError: amazonPayment.onError,
				onAddressSelect: function() {
					jQuery('#amazon-confirm-payment').prop('disabled', true);
				}
			};
			amazonPayment.addressBook = new OffAmazonPayments.Widgets.AddressBook(addressBookParameter).bind('amazon-address-book');
		}
	},

	/**
	 * Show the wallet widget
	 */
	showWallet: function() {
		if (jQuery('#amazon-wallet')) {
			Logger.log('showWallet: ', amazonPayment.orderReferenceId);
			var walletParameter = {
				sellerId: amazonPayment.config.sellerId,
				design: {
					designMode: 'responsive'
				},
				//onReady: function(orderReference) {
				//	Logger.log('wallet.onReady', orderReference);
					// Initiate the payment in backend
				//	amazonPayment.initializePayment();
				//},
				onError: amazonPayment.onError,
				onPaymentSelect: function() {
					jQuery('#amazon-confirm-payment').prop('disabled', false);
				}
			};
			// Don't use onOrderReferenceCreate in case of using addresses
			if(!amazonPayment.config.addressRequired) {
				walletParameter.onOrderReferenceCreate = amazonPayment.onOrderReferenceCreate;
			}
			amazonPayment.wallet = new OffAmazonPayments.Widgets.Wallet(walletParameter).bind("amazon-wallet");
		}
	},

	/**
	 * Scroll to a selector position
	 * @param selector
	 */
	scrollTo: function(selector) {
		var element = jQuery(selector);
		// In case of an unavailable selector
		if(element.length===0) {
			// Use the cars wrapper
			element = jQuery('.tx-amazonpay');
		}
		jQuery('html, body').animate({
			scrollTop: element.offset().top - 120
		}, 500);
	},

	/**
	 * Toggle loading info
	 */
	toggleLoadingInfo: function() {
		var loading = jQuery('#amazon-payment-loading');
		if(loading.css('display') === 'none') {
			loading.fadeIn(100);
		}
		else {
			loading.fadeOut(300);
		}
	},

	/**
	 * Hide loading info
	 */
	hideLoadingInfo: function() {
		var loading = jQuery('#amazon-payment-loading');
		if(loading.css('display') !== 'none') {
			loading.fadeOut(300);
		}
	}
};

/**
 * On amazon login ready.
 * Amazon-Login button is ready to use!
 * Set client id
 */
window.onAmazonLoginReady = function() {
	Logger.log('onAmazonLoginReady', amazonPayment.config);
	// Set the client id
	amazon.Login.setClientId(amazonPayment.config.clientId);
	// Hide loading info
	amazonPayment.toggleLoadingInfo();
};

/**
 * On amazon payments ready.
 * Amazon payments is ready to receive a call by a successfully login!
 * Create the required widgets
 */
window.onAmazonPaymentsReady = function() {
	Logger.log('onAmazonPaymentsReady', amazonPayment.config);
	OffAmazonPayments.Button('amazon-login', amazonPayment.config.sellerId, {
		type: amazonPayment.config.login.type,
		color: amazonPayment.config.login.color,
		size: amazonPayment.config.login.size,
		language: amazonPayment.config.login.language,
		authorization: function() {
			Logger.log('OffAmazonPayments.Button.authorization()');
			// Login options
			loginOptions = {
				scope: amazonPayment.config.login.scope,
				popup: amazonPayment.config.login.popup
			};
			// Without redirect URL
			if(amazonPayment.config.login.redirectUrl === '') {
				amazonPayment.authRequest = amazon.Login.authorize(loginOptions, function(response) {
					Logger.log('OffAmazonPayments.Button.authorization() response:', response);
					amazonPayment.toggleLoadingInfo();
					if(response.status == 'complete' && typeof response.access_token != 'undefined') {
						//
						amazonPayment.addressConsentToken = response.access_token;
						// Create an address book instance
						if(amazonPayment.config.addressRequired) {
							amazonPayment.showAddressBook();
						}
						// Create a wallet instance
						amazonPayment.showWallet();
						// Remind login
						amazonPayment.setLoggedIn(true);
						// Display an success message
						/**
						 * @todo translate message
						 * TYPO3 7.6 required!
						 */
						FlashMessage.push('Successfully logged in', 'success', '#amazon-flash-messages');
					}
				});
			}
			else {
				amazonPayment.authRequest = amazon.Login.authorize(loginOptions, amazonPayment.config.login.redirectUrl);
			}
			Logger.log('OffAmazonPayments.Button.authorization() authRequest: ', amazonPayment.authRequest);
		},
		onError: amazonPayment.onError
	});
	
};